@@ -3,7 +3,7 @@
 # Executes all scripts located in ~/.config/pianobar/eventcmd.d/ as if they
 # were called by pianobar directly
 
-STDIN=`mktemp`
+STDIN=`mktemp ${TMPDIR:-/tmp}/pianobar.XXXXXX`
 cat >> $STDIN
 
 for F in ~/.config/pianobar/eventcmd.d/*; do
