@@ -21,10 +21,7 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 */
 
-#ifndef __FreeBSD__
-#define _BSD_SOURCE /* required by strdup() */
-#define _DARWIN_C_SOURCE /* strdup() on OS X */
-#endif
+#include "../config.h"
 
 #include <json.h>
 #include <string.h>
@@ -411,6 +408,7 @@ PianoReturn_t PianoResponse (PianoHandle_t *ph, PianoRequest_t *req) {
 		case PIANO_REQUEST_BOOKMARK_ARTIST:
 		case PIANO_REQUEST_DELETE_FEEDBACK:
 		case PIANO_REQUEST_DELETE_SEED:
+		case PIANO_REQUEST_CHANGE_SETTINGS:
 			/* response unused */
 			break;
 
@@ -508,6 +506,17 @@ PianoReturn_t PianoResponse (PianoHandle_t *ph, PianoRequest_t *req) {
 			break;
 		}
 
+		case PIANO_REQUEST_GET_SETTINGS: {
+			PianoSettings_t * const settings = req->data;
+
+			assert (settings != NULL);
+
+			settings->explicitContentFilter = json_object_get_boolean (
+					json_object_object_get (result, "isExplicitContentFilterEnabled"));
+			settings->username = PianoJsonStrdup (result, "username");
+			break;
+		}
+
 		case PIANO_REQUEST_GET_STATION_INFO: {
 			/* get station information (seeds and feedback) */
 			PianoRequestDataGetStationInfo_t *reqData = req->data;
@@ -568,7 +577,14 @@ PianoReturn_t PianoResponse (PianoHandle_t *ph, PianoRequest_t *req) {
 			json_object *feedback = json_object_object_get (result,
 					"feedback");
 			if (feedback != NULL) {
-				json_object_object_foreach (feedback, key, val) {
+				static const char * const keys[] = {"thumbsUp", "thumbsDown"};
+				for (size_t i = 0; i < sizeof (keys)/sizeof (*keys); i++) {
+					json_object * const val = json_object_object_get (feedback,
+							keys[i]);
+					if (val == NULL) {
+						continue;
+					}
+					assert (json_object_is_type (val, json_type_array));
 					for (int i = 0; i < json_object_array_length (val); i++) {
 						json_object *s = json_object_array_get_idx (val, i);
 						PianoSong_t *feedbackSong;
