@@ -30,16 +30,15 @@ THE SOFTWARE.
 #include <sys/types.h>
 #include <pthread.h>
 #include <stdint.h>
+#include <signal.h>
 
 #include <ao/ao.h>
 #include <libavformat/avformat.h>
 #include <libavfilter/avfilter.h>
 #include <libavfilter/avfiltergraph.h>
 #include <piano.h>
-#include <waitress.h>
 
 #include "settings.h"
-#include "config.h"
 
 typedef struct {
 	/* protected by pauseMutex */
@@ -49,9 +48,13 @@ typedef struct {
 	pthread_cond_t pauseCond;
 
 	enum {
-		PLAYER_DEAD = 0, /* thread is not running */
-		PLAYER_STARTING, /* thread is starting */
+		/* not running */
+		PLAYER_DEAD = 0,
+		/* running, but not ready to play music yet */
+		PLAYER_WAITING,
+		/* currently playing a song */
 		PLAYER_PLAYING,
+		/* finished playing a song */
 		PLAYER_FINISHED,
 	} mode;
 
@@ -63,14 +66,11 @@ typedef struct {
 	AVFilterContext *fbufsink, *fabuf;
 	int streamIdx;
 	int64_t lastTimestamp;
-#ifndef HAVE_AV_TIMEOUT
-	int64_t ping;
-#endif
+	sig_atomic_t interrupted;
 
 	ao_device *aoDev;
 
 	/* settings */
-	volatile double volume;
 	double gain;
 	char *url;
 	const BarSettings_t *settings;
