@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2008-2012
+Copyright (c) 2008-2015
 	Lars-Dominik Braun <lars@6xq.net>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -23,11 +23,7 @@ THE SOFTWARE.
 
 /* application settings */
 
-#ifndef __FreeBSD__
-#define _POSIX_C_SOURCE 1 /* PATH_MAX */
-#define _BSD_SOURCE /* strdup() */
-#define _DARWIN_C_SOURCE /* strdup() on OS X */
-#endif
+#include "config.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -36,11 +32,14 @@ THE SOFTWARE.
 #include <assert.h>
 #include <sys/types.h>
 #include <pwd.h>
+#include <unistd.h>
+#include <ctype.h>
 
 #include <piano.h>
 
 #include "settings.h"
 #include "config.h"
+#include "ui.h"
 #include "ui_dispatch.h"
 
 #define streq(a, b) (strcmp (a, b) == 0)
@@ -177,9 +176,6 @@ void BarSettingsRead (BarSettings_t *settings) {
 	settings->outkey = strdup ("6#26FRL$ZWD");
 	settings->fifo = BarGetXdgConfigDir (PACKAGE "/ctl");
 	assert (settings->fifo != NULL);
-	memcpy (settings->tlsFingerprint, "\x2D\x0A\xFD\xAF\xA1\x6F\x4B\x5C\x0A"
-			"\x43\xF3\xCB\x1D\x47\x52\xF9\x53\x55\x07\xC0",
-			sizeof (settings->tlsFingerprint));
 
 	settings->msgFormat[MSG_NONE].prefix = NULL;
 	settings->msgFormat[MSG_NONE].postfix = NULL;
@@ -203,8 +199,10 @@ void BarSettingsRead (BarSettings_t *settings) {
 	/* read config files */
 	for (size_t j = 0; j < sizeof (configfiles) / sizeof (*configfiles); j++) {
 		static const char *formatMsgPrefix = "format_msg_";
-		char key[256], val[256];
 		FILE *configfd;
+		/* getline allocates these on the first run */
+		char *line = NULL;
+		size_t lineLen = 0, lineNum = 0;
 
 		char * const path = BarGetXdgConfigDir (configfiles[j]);
 		assert (path != NULL);
@@ -214,14 +212,59 @@ void BarSettingsRead (BarSettings_t *settings) {
 		}
 
 		while (1) {
-			char lwhite, rwhite;
-			int scanRet = fscanf (configfd, "%255s%c=%c%255[^\n]", key, &lwhite, &rwhite, val);
-			if (scanRet == EOF) {
+			++lineNum;
+			ssize_t ret = getline (&line, &lineLen, configfd);
+			if (ret == -1) {
+				/* EOF or error */
 				break;
-			} else if (scanRet != 4 || lwhite != ' ' || rwhite != ' ') {
-				/* invalid config line */
+			}
+			/* parse lines that match "^\s*(.*?)\s?=\s?(.*)$". Windows and Unix
+			 * line terminators are supported. */
+			char *key = line;
+
+			/* skip leading spaces */
+			while (isspace ((unsigned char) key[0])) {
+				++key;
+			}
+
+			/* skip comments */
+			if (key[0] == '#') {
 				continue;
 			}
+
+			/* search for delimiter and split key-value pair */
+			char *val = strchr (line, '=');
+			if (val == NULL) {
+				/* no warning for empty lines */
+				if (key[0] != '\0') {
+					BarUiMsg (settings, MSG_INFO,
+							"Invalid line at %s:%zu\n", path, lineNum);
+				}
+				/* invalid line */
+				continue;
+			}
+			*val = '\0';
+			++val;
+
+			/* drop spaces at the end */
+			char *keyend = &key[strlen (key)-1];
+			while (keyend >= key && isspace ((unsigned char) *keyend)) {
+				*keyend = '\0';
+				--keyend;
+			}
+
+			/* strip at most one space, legacy cruft, required for values with
+			 * leading spaces like love_icon */
+			if (isspace ((unsigned char) val[0])) {
+				++val;
+			}
+			/* drop trailing cr/lf */
+			char *valend = &val[strlen (val)-1];
+			while (valend >= val && (*valend == '\r' || *valend == '\n')) {
+				*valend = '\0';
+				--valend;
+			}
+
 			if (streq ("control_proxy", key)) {
 				settings->controlProxy = strdup (val);
 			} else if (streq ("proxy", key)) {
@@ -253,6 +296,9 @@ void BarSettingsRead (BarSettings_t *settings) {
 			} else if (streq ("decrypt_password", key)) {
 				free (settings->inkey);
 				settings->inkey = strdup (val);
+			} else if (streq ("ca_bundle", key)) {
+				free (settings->caBundle);
+				settings->caBundle = strdup (val);
 			} else if (memcmp ("act_", key, 4) == 0) {
 				size_t i;
 				/* keyboard shortcuts */
@@ -278,7 +324,7 @@ void BarSettingsRead (BarSettings_t *settings) {
 				free (settings->autostartStation);
 				settings->autostartStation = strdup (val);
 			} else if (streq ("event_command", key)) {
-				settings->eventCmd = strdup (val);
+				settings->eventCmd = BarSettingsExpandTilde (val, userhome);
 			} else if (streq ("history", key)) {
 				settings->history = atoi (val);
 			} else if (streq ("max_player_errors", key)) {
@@ -323,16 +369,6 @@ void BarSettingsRead (BarSettings_t *settings) {
 				settings->fifo = BarSettingsExpandTilde (val, userhome);
 			} else if (streq ("autoselect", key)) {
 				settings->autoselect = atoi (val);
-			} else if (streq ("tls_fingerprint", key)) {
-				/* expects 40 byte hex-encoded sha1 */
-				if (strlen (val) == 40) {
-					for (size_t i = 0; i < 20; i++) {
-						char hex[3];
-						memcpy (hex, &val[i*2], 2);
-						hex[2] = '\0';
-						settings->tlsFingerprint[i] = strtol (hex, NULL, 16);
-					}
-				}
 			} else if (strncmp (formatMsgPrefix, key,
 					strlen (formatMsgPrefix)) == 0) {
 				static const char *mapping[] = {"none", "info", "nowplaying",
@@ -363,11 +399,15 @@ void BarSettingsRead (BarSettings_t *settings) {
 						break;
 					}
 				}
+			} else {
+				BarUiMsg (settings, MSG_INFO,
+						"Unrecognized key %s at %s:%zu\n", key, path, lineNum);
 			}
 		}
 
 		fclose (configfd);
 		free (path);
+		free (line);
 	}
 
 	/* check environment variable if proxy is not set explicitly */
@@ -378,6 +418,11 @@ void BarSettingsRead (BarSettings_t *settings) {
 		}
 	}
 
+	/* ffmpeg does not support setting an http proxy explicitly */
+	if (settings->proxy != NULL) {
+		setenv ("http_proxy", settings->proxy, 1);
+	}
+
 	free (userhome);
 }
 
