@@ -27,7 +27,6 @@ THE SOFTWARE.
 #include <stdbool.h>
 
 #include <piano.h>
-#include <waitress.h>
 
 /* update structure in ui_dispatch.h if you add shortcuts here */
 typedef enum {
@@ -60,8 +59,9 @@ typedef enum {
 	BAR_KS_PLAY = 26,
 	BAR_KS_PAUSE = 27,
 	BAR_KS_VOLRESET = 28,
+	BAR_KS_SETTINGS = 29,
 	/* insert new shortcuts _before_ this element and increase its value */
-	BAR_KS_COUNT = 29,
+	BAR_KS_COUNT = 30,
 } BarKeyShortcutId_t;
 
 #define BAR_KS_DISABLED '\x00'
@@ -102,8 +102,7 @@ typedef struct {
 	char *npStationFormat;
 	char *listSongFormat;
 	char *fifo;
-	char *rpcHost, *rpcTlsPort, *partnerUser, *partnerPassword, *device, *inkey, *outkey;
-	char tlsFingerprint[20];
+	char *rpcHost, *rpcTlsPort, *partnerUser, *partnerPassword, *device, *inkey, *outkey, *caBundle;
 	char keys[BAR_KS_COUNT];
 	BarMsgFormatStr_t msgFormat[MSG_COUNT];
 } BarSettings_t;
