@@ -27,7 +27,6 @@ THE SOFTWARE.
 #include <stdbool.h>
 
 #include <piano.h>
-#include <waitress.h>
 
 #include "settings.h"
 #include "player.h"
@@ -49,10 +48,10 @@ void BarUiPrintSong (const BarSettings_t *, const PianoSong_t *,
 		const PianoStation_t *);
 size_t BarUiListSongs (const BarSettings_t *, const PianoSong_t *, const char *);
 void BarUiStartEventCmd (const BarSettings_t *, const char *,
-		const PianoStation_t *, const PianoSong_t *, const struct audioPlayer *,
-		PianoStation_t *, PianoReturn_t, WaitressReturn_t);
-int BarUiPianoCall (BarApp_t * const, PianoRequestType_t,
-		void *, PianoReturn_t *, WaitressReturn_t *);
+		const PianoStation_t *, const PianoSong_t *, const player_t *,
+		PianoStation_t *, PianoReturn_t, CURLcode);
+bool BarUiPianoCall (BarApp_t * const, const PianoRequestType_t,
+		void *, PianoReturn_t *, CURLcode *);
 void BarUiHistoryPrepend (BarApp_t *app, PianoSong_t *song);
 
 #endif /* SRC_UI_H_46P20TS0 */
